//This module is a part of FoxUtils source code

unit Stdout;

interface

uses
	Windows, SysUtils, StrUtils, ShellApi;

resourcestring sSetStdHandleMsg = 'Error, when setting Handle.'+Chr(13)+Chr(13)+'SetStdHandle(STD_INPUT_HANDLE, Value)';
resourcestring sCreatePipeMsg = 'Error when creating - Pipe.'+Chr(13)+Chr(13)+'CreatePipe(FChildStdoutRd, FChildStdoutWr, saAttr, 0)';
resourcestring sDuplicateHandleMsg = 'Error when duplicate Handle Msg.'+Chr(13)+Chr(13)+'DuplicateHandle(GetCurrentProcess(), FChildStdoutRd, GetCurrentProcess(), @FChildStdoutRdDup, 0, False, DUPLICATE_SAME_ACCESS)';
resourcestring sCreateChildProcessMsg = 'Error creating Chilg Process.'+Chr(13)+Chr(13)+'CreateChildProcess(ExeName, CommadLine, FChildStdoutWr)';


type

  ESetStdHandleErr = class(Exception);
  ECreatePipeErr = class(Exception);
  EDuplicateHandleErr = class(Exception);
  ECreateChildProcessErr = class(Exception);

{ THandledObject }

{ This is a generic class for all encapsulated WinAPI's which need to call
  CloseHandle when no longer needed.  This code eliminates the need for
  3 identical destructors in the TEvent, TMutex, and TSharedMem classes
  which are descended from this class. }

  THandledObject = class(TObject)
  protected
    FHandle: THandle;
  public
    destructor Destroy; override;
    property Handle: THandle read FHandle;
  end;

{ TMutex }

{ This class encapsulates the concept of a Win32 mutex.  See "CreateMutex"
  in the Win32 reference for more information }

  TMutex = class(THandledObject)
  public
    constructor Create(const Name: string);
    function Get(TimeOut: Integer): Boolean;
    function Release: Boolean;
  end;

{ TChildProc }
  TChildProc = class(TObject)
    FStdIn: THandle;
    FStdOut: THandle;
    FsaAttr: PSecurityAttributes;
    FChildStdoutRd: THandle;
    FChildStdoutWr: THandle;
    FChildStdinRd: THandle;
    FChildStdinWr: THandle;
  private
    function CreateChildProcess(ExeName, CommadLine: String; StdIn: THandle; StdOut: THandle): Boolean;
  public
    constructor Create(ExeName, CommadLine: String);
    destructor Destroy; override;
    function ReadStrFromChild(Timeout: Integer=1000): String;
  end;


implementation

{ THandledObject }

destructor THandledObject.Destroy;
begin
  if FHandle <> 0 then
    CloseHandle(FHandle);
end;

{ TMutex }

constructor TMutex.Create(const Name: string);
begin
  FHandle := CreateMutex(nil, False, PChar(Name));
  if FHandle = 0 then abort;
end;

function TMutex.Get(TimeOut: Integer): Boolean;
begin
  Result := WaitForSingleObject(FHandle, TimeOut) = WAIT_OBJECT_0;
end;

function TMutex.Release: Boolean;
begin
  Result := ReleaseMutex(FHandle);
end;

{ TChildProc }

constructor TChildProc.Create(ExeName, CommadLine: String);
Var
  Tmp1, Tmp2: THandle;
begin
  New(FsaAttr);
  Try
    FsaAttr.nLength:=SizeOf(SECURITY_ATTRIBUTES);
    FsaAttr.bInheritHandle:=True;
    FsaAttr.lpSecurityDescriptor:=Nil;
    If not CreatePipe(FChildStdoutRd, FChildStdoutWr, FsaAttr, 0) Then  // "" Pipe
      raise ECreatePipeErr.CreateRes(@sCreatePipeMsg)
    Else
      If not CreatePipe(FChildStdinRd, FChildStdinWr, FsaAttr, 0) Then  // "" Pipe
        raise ECreatePipeErr.CreateRes(@sCreatePipeMsg)
      Else
        begin

			  	If not DuplicateHandle(GetCurrentProcess(), FChildStdoutRd, GetCurrentProcess(), @Tmp1, 0, False, DUPLICATE_SAME_ACCESS) Then
            raise EDuplicateHandleErr.CreateRes(@sDuplicateHandleMsg)
     	    Else
  			  	If not DuplicateHandle(GetCurrentProcess(), FChildStdinWr, GetCurrentProcess(), @Tmp2, 0, False, DUPLICATE_SAME_ACCESS) Then
              raise EDuplicateHandleErr.CreateRes(@sDuplicateHandleMsg)
            Else
              begin
                CloseHandle(FChildStdoutRd);
                CloseHandle(FChildStdinWr);
                FChildStdoutRd:=Tmp1;
                FChildStdinWr:=Tmp2;
               	If not CreateChildProcess(ExeName, CommadLine, FChildStdinRd, FChildStdoutWr) Then  
                  raise ECreateChildProcessErr.CreateRes(@sCreateChildProcessMsg)
              end;
        end;
  Finally
    Dispose(FsaAttr);
  End;
end;

function TChildProc.CreateChildProcess(ExeName, CommadLine: String; StdIn,
  StdOut: THandle): Boolean;
Var
  piProcInfo: TProcessInformation;
  siStartInfo: TStartupInfo;
begin
// Set up members of STARTUPINFO structure.
  ZeroMemory(@siStartInfo, SizeOf(TStartupInfo));
  siStartInfo.cb:=SizeOf(TStartupInfo);
  siStartInfo.hStdInput:=StdIn;
  siStartInfo.hStdOutput:=StdOut;
	siStartInfo.dwFlags:=STARTF_USESTDHANDLES+STARTF_USESHOWWINDOW;
  siStartInfo.wShowWindow := SW_HIDE;
// Create the child process.
   Result:=CreateProcess(Nil,
      PChar(ExeName+' '+CommadLine),       // command line
      Nil,          // process security attributes
      Nil,          // primary thread security attributes
      TRUE,         // handles are inherited
      0,            // creation flags
      Nil,          // use parent's environment
      Nil,          // use parent's current directory
      siStartInfo,  // STARTUPINFO pointer
      piProcInfo);  // receives PROCESS_INFORMATION
end;

destructor TChildProc.Destroy;
begin

  CloseHandle(FChildStdoutRd);
  CloseHandle(FChildStdoutWr);
  CloseHandle(FChildStdinRd);
  CloseHandle(FChildStdinWr);
end;

function TChildProc.ReadStrFromChild(Timeout: Integer): String;
Var
	i: Integer;
	dwRead, BufSize, DesBufSize: DWORD;
  chBuf: PChar;
  Res: Boolean;
begin
	Try
		BufSize:=0;
    New(chBuf);
    Repeat
      For i:=0 to 9 do
  	    begin
			    Res:=PeekNamedPipe(FChildStdoutRd, nil, 0, nil, @DesBufSize, nil);
      	  Res:=Res and (DesBufSize > 0);
          If Res Then
            Break;
          Sleep(Round(Timeout/10));
        end;
      If Res Then
      	begin
        	If DesBufSize > BufSize Then
          	begin
            	FreeMem(chBuf);
	            GetMem(chBuf, DesBufSize);
              BufSize:=DesBufSize;
            end;
	  			Res:=ReadFile(FChildStdoutRd, chBuf^, BufSize, dwRead, Nil);
     			Result:=Result+LeftStr(chBuf, dwRead);
        end;
    Until not Res;
  Except
  	Result:='Read Err';
  End;
end;

end.
